// Ryan Schmidt
// Copyright (c) 2006-2010
// Distributed under the Boost Software License, Version 1.0.
// http://www.boost.org/LICENSE_1_0.txt
// http://www.geometrictools.com/License/Boost/LICENSE_1_0.txt

#pragma once

#include <VFTriangleMesh.h>
#include <ExpMapGenerator.h>
#include <IMeshBVTree.h>
#include "Texture.h"

namespace Wml {
	class ExtendedWmlCamera;
}


class MeshObject
{
public:
	MeshObject(void);
	~MeshObject(void);

	void ReadMeshOBJ( const char * pFilename );
	void SetMesh( rms::VFTriangleMesh & mesh );

	void Render(bool bDrawWireframe = false, bool bDrawNormals = false);
	void RenderParamMesh();

	rms::VFTriangleMesh & GetMesh() { return m_mesh; }

	bool FindIntersection( Wml::Ray3f & vRay, Wml::Vector3f & vHit, Wml::Vector3f & vHitNormal );

	rms::Frame3f & ExpMapFrame() { return m_vSeedFrame; }
	void MoveExpMap( Wml::Ray3f & vRay );
	void ScaleExpMap( float fScale );
	void RotateExpMap( float fRotate );
	void ValidateExpMap();

	void MoveExpMap2( Wml::Ray3f & vRay );
	void AlignSeedFrame2(Wml::ExtendedWmlCamera * pCamera);

	bool GetSmoothNormals() const { return m_bSmoothNormals; }
	void SetSmoothNormals( bool bSet );

	bool GetUpwindAverage() const { return m_bUpwindAverage; }
	void SetUpwindAverage( bool bSet );

	bool GetUseConstantNormal() const { return m_bUseConstantNormal; }
	void SetUseConstantNormal( bool bSet );

	bool GetPreserveProjectedLengths() const { return m_bPreserveProjectedLengths; }
	void SetPreserveProjectedLengths( bool bSet );

	void SmoothNormals(int nCount = 1);

protected:
	rms::VFTriangleMesh m_mesh;
	rms::IMeshBVTree m_bvTree;
	float m_fMaxEdgeLength;

	rms::Frame3f m_vSeedFrame;
	float m_fDecalRadius;
	float m_fBoundaryWidth;
	bool m_bSmoothNormals;
	bool m_bUpwindAverage;
	bool m_bUseConstantNormal;
	bool m_bPreserveProjectedLengths;

	rms::ExpMapGenerator m_expmapgen;
	bool m_bExpMapInitialized;
	bool m_bExpMapValid;

	rms::Frame3f m_vSeedFrame2;
	bool m_bExpMap2Initialized;
	rms::VFTriangleMesh m_mesh2;

	Texture m_texture;
};
